/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor;

/**
 * Represents position and number of available fences of a player on
 * a {@link Board}.
 */
public final class Player extends Object {

    final int x;
    final int y;
    /** number of fences this player has */
    final int f;
    /** the direction of players end line */
    final Direction endDirection;

    Player(int x, int y, int f, Player.Direction endDir) {
        super();
        this.x = x;
        this.y = y;
        this.f = f;
        this.endDirection = endDir;
    }

    /** Returns the x-coordinate of the player.
     * @return  number from 0 to 16
     */
    int getXInternal() {
        return x;
    }

    /** Returns the y-coordinate of the player.
     * @return  number from 0 to 16
     */
    int getYInternal() {
        return y;
    }

    /** Returns position of the player from 0-8.
     *
     * @return integer from 0-8
     */
    public int getColumn() {
        return x / 2;
    }

    /** The y-position of the player.
     * @return integer from 0-8
     */
    public int getRow() {
        return y / 2;
    }

    /** How much fences is still available for the player.
     * 
     * @return number of fences this player still has
     */
    public int getFences() {
        return f;
    }

    @Override
    public String toString() {
        return "Player[" + getColumn() + "," + getRow() + "," + f + "," + endDirection + "]";
    }

    @Override
    public int hashCode() {
        return 8 * x + 2 * y + 4 * f + 7 + endDirection.hashCode();
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof Player) {
            Player p = (Player) o;
            return x == p.x && y == p.y && f == p.f && endDirection.equals(p.endDirection);
        }
        return false;
    }

    /** Possible directions of player's moves.
     */
    public enum Direction {

        NORTH('P', 16), WEST((char) -1, -1), EAST((char) -1, -1), SOUTH('Q', 0);
        final char player;
        final int finalLine;

        private Direction(char ch, int fl) {
            this.player = ch;
            this.finalLine = fl;
        }

        final boolean reached(Player p) {
            return p.getYInternal() == finalLine;
        }

        static Direction valueOf(char ch) throws IllegalPositionException {
            for (Direction direction : values()) {
                if (direction.name().charAt(0) == ch) {
                    return direction;
                }
            }
            throw new IllegalPositionException("No direction " + ch);
        }
    }
}
